/* Terraform - (C) 1997-1999 Robert Gasch (Robert_Gasch@peoplesoft.com)
 *  - www.peoplesoft.com/peoplepages/g/robert_gasch/terraform/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "TFDialogFold.h"
#include "GuiDialogOAC.h"
#include "GlobalSanityCheck.h"
#include "GlobalTrace.h"



/*
 *  constructor: initialize all data members 
 */
TFDialogFold::TFDialogFold (HeightField *HF, HeightFieldDraw *HFD)
	        : TFPreviewDialog (HF, HFD, "Terraform Fold Dialog", 
				"Edge Fold Offset", 6, 4),
		  d_hBoxFoldOffset (TRUE, 0),
		  d_vBoxOptions (TRUE, 5),
		  d_adjFoldOffset (0.1, 0.01, 0.4, 0.01),
		  d_hsFoldOffset (d_adjFoldOffset),
		  d_lblFoldOffset ("Edge Fold Offset"),
		  d_optionsFrame ("Edge Fold Options") 

{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "+++ TFDialogFold\n");

	buildDialogWindow ();
	this->setHFobjs (HF, HFD);
	this->iterateEvents ();
	updatePreviewCallback ();

	connect_to_method (d_adjFoldOffset.value_changed, this, &updatePreviewCallback);
}


/*
 *  destructor: clean up 
 */
TFDialogFold::~TFDialogFold ()
{
	GlobalTrace::trace (GlobalTrace::TRACE_FLOW, "--- TFDialogFold\n");
}


/*
 *  fillVBox: fill the dialog's VBox
 */
void TFDialogFold::insertOptions ()
{
	SanityCheck::bailout ((!this->d_baseFrame), "TFDialogFold called without Frame label",
		"TFDialogFold::setHFobjs");

	d_optionsFrame.set_shadow_type (GTK_SHADOW_ETCHED_IN);
	d_optionsFrame.set_border_width (5);

	d_hBoxFoldOffset.pack_start (d_lblFoldOffset, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_lblFoldOffset.show ();
	d_hsFoldOffset.set_update_policy (GTK_UPDATE_CONTINUOUS);
	d_hsFoldOffset.set_digits (2);
	d_hsFoldOffset.set_draw_value (TRUE);
	d_hBoxFoldOffset.pack_end (d_hsFoldOffset, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hsFoldOffset.show ();
	d_vBoxOptions.pack_start (d_hBoxFoldOffset, TRUE, TRUE, GuiDialogOAC::s_HBOff);
	d_hBoxFoldOffset.show ();

	d_optionsFrame.add (d_vBoxOptions);
	this->d_table->attach (d_optionsFrame, 3, 6, 1, 3);
	d_vBoxOptions.show ();
	d_optionsFrame.show ();
	this->d_baseFrame->show ();
	this->get_vbox()->show ();
}


/*
 *  buildDialogWindow:
 */
void TFDialogFold::buildDialogWindow ()
{
	this->set_usize (450, 225);
	this->set_title (d_winTitle);

	this->get_vbox()->set_border_width (2);

	this->setupVBox ();
	this->insertPreview ();
	insertOptions();
	this->fillActionArea ();

	this->show ();
}


/*
 *  buttonCallbackApply: fold the height field 
 */
void TFDialogFold::buttonCallbackApply ()
{
	float		value = d_adjFoldOffset.get_value ();

	SanityCheck::bailout ((!d_HFO), "d_HFO==NULL", "TFDialogFold::buttonCallbackApply");
	d_HFO->fold ((int)(value*
			(d_HF->getWidth()+d_HF->getHeight())/2));
	d_HFD->draw ();

	// only do this if window stays open 
	if (d_applyHit)
		{
		this->setHFobjs (d_HF, d_HFD);
		updatePreviewCallback ();
		}

	this->d_applyHit = TRUE;
}


/*
 *  updatePreviewCallback: update the preview after a widget has been updated.
 */
void TFDialogFold::updatePreviewCallback ()
{
	if (!d_cbUsePreview.get_active())
		return;

	float		value = d_adjFoldOffset.get_value ();

	SanityCheck::bailout ((!d_HFPreview), "d_HFPreview==NULL", "TFDialogFold");
        SanityCheck::bailout ((!d_HFOPreview), "d_HFOPreview==NULL", "TFDialogFold");
        SanityCheck::bailout ((!d_HFDPreview), "d_HFDPreview==NULL", "TFDialogFold");
	this->previewUpToDate ();
	d_HFPreview->restore ();
	d_HFOPreview->fold ((int)(value*
			(d_HFPreview->getWidth()+d_HFPreview->getHeight())/2));
	d_HFDPreview->setColormap (d_HFD->getColormap());
	d_HFDPreview->draw ();
}

